@@ -16,7 +16,15 @@
  */
 package org.osgi.util.converter;
 
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Ignore;
+import org.junit.Test;
+import org.osgi.util.converter.ConverterMapTest.TestValue;
+
 import java.lang.annotation.Annotation;
+import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
 import java.lang.reflect.Method;
 import java.math.BigInteger;
 import java.net.URL;
@@ -29,11 +37,6 @@ import java.util.Hashtable;
 import java.util.LinkedHashMap;
 import java.util.Map;
 
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Ignore;
-import org.junit.Test;
-
 import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
@@ -179,6 +182,38 @@ public class ConverterMapTest {
         assertEquals(true, (boolean) m.get("za.za"));
     }
 
+    @Test
+    public void testInterfaceToMapEmptySub() {
+        TestInterfaceSub impl = new TestInterfaceSub() {
+            @Override
+            public String foo() {
+                return "Chocolate!";
+            }
+
+            @Override
+            public int bar() {
+                return 76543;
+            }
+
+            @Override
+            public int bar(String def) {
+                return 0;
+            }
+
+            @Override
+            public Boolean za_za() {
+                return true;
+            }
+        };
+
+        @SuppressWarnings("rawtypes")
+        Map m = converter.convert(impl).to(Map.class);
+        assertEquals(3, m.size());
+        assertEquals("Chocolate!", m.get("foo"));
+        assertEquals(76543, (int) m.get("bar"));
+        assertEquals(true, (boolean) m.get("za.za"));
+    }
+
     @SuppressWarnings({ "unchecked", "rawtypes" })
     @Test
     public void testMapToInterface1() {
@@ -551,6 +586,34 @@ public class ConverterMapTest {
         assertEquals("987", m2.get("continue"));
     }
 
+    @Test
+    public void testSingleElementAnnotationPrefix() {
+        final Converter converter = Converters.standardConverter();
+        final TestValue testValue = converter.convert(Collections.singletonMap("my.prefix.test.value", true)).to(TestValue.class);
+        assertTrue(testValue.value());
+    }
+
+    @Test
+    @TestValue(true)
+    public void testSingleElementAnnotationPrefixToMap() throws Exception {
+        final Converter converter = Converters.standardConverter();
+        Method method = getClass().getMethod("testSingleElementAnnotationPrefixToMap");
+        TestValue annotation = method.getDeclaredAnnotation(TestValue.class);
+        Map<String, Object> map = converter.convert(annotation).to(new TypeReference<Map<String, Object>>() {});
+        assertTrue((Boolean)map.get("my.prefix.test.value"));
+    }
+
+    @Test
+    @PrefixMarkerAnnotation
+    public void testMarkerAnnotationPrefixToMap() throws Exception {
+        final Converter converter = Converters.standardConverter();
+        Method method = getClass().getMethod("testMarkerAnnotationPrefixToMap");
+        PrefixMarkerAnnotation annotation = method.getDeclaredAnnotation(PrefixMarkerAnnotation.class);
+        Map<String, Object> map = converter.convert(annotation).to(new TypeReference<Map<String, Object>>() {});
+        assertTrue(map.containsKey("org.foo.bar.prefix.marker.annotation"));
+        assertTrue((Boolean)map.get("org.foo.bar.prefix.marker.annotation"));
+    }
+
     private <K,V> Map.Entry<K,V> getMapEntry(Map<K,V> map) {
         assertEquals("This method assumes a map of size 1", 1, map.size());
         return map.entrySet().iterator().next();
@@ -563,6 +626,8 @@ public class ConverterMapTest {
         Boolean za_za();
     }
 
+    interface TestInterfaceSub extends TestInterface {}
+
     interface TestInterfaceWithGetProperties {
         int blah();
         Dictionary<String, Object> getProperties();
@@ -579,6 +644,13 @@ public class ConverterMapTest {
     	long somethingElse() default -87;
     }
 
+    @Retention(RetentionPolicy.RUNTIME)
+    @interface TestValue {
+        static final String PREFIX_ = "my.prefix.";
+
+        boolean value();
+    }
+
     private static class Foo {
         private final int value;
 
