@@ -23,6 +23,7 @@ import org.osgi.framework.Version;
 import org.osgi.util.converter.MyDTO.Count;
 import org.osgi.util.converter.MyEmbeddedDTO.Alpha;
 
+import java.lang.reflect.Array;
 import java.lang.reflect.Type;
 import java.math.BigDecimal;
 import java.math.BigInteger;
@@ -342,6 +343,52 @@ public class ConverterTest {
         assertArrayEquals(new char [] {'x', 'y'}, ca4b);
         assertNotSame("Should have created a new instance", ca4a, ca4b);
     }
+    
+    /**
+     * 707.4.3.1 - null becomes an empty array
+     */
+    @Test
+    public void testNullToArrayConversion() {
+    	
+    	checkArray(String[].class);
+    	checkArray(boolean[].class);
+    	checkArray(byte[].class);
+    	checkArray(short[].class);
+    	checkArray(char[].class);
+    	checkArray(int[].class);
+    	checkArray(float[].class);
+    	checkArray(long[].class);
+    	checkArray(double[].class);
+
+    	checkArray(String[][].class);
+    	checkArray(boolean[][].class);
+    	checkArray(byte[][].class);
+    	checkArray(short[][].class);
+    	checkArray(char[][].class);
+    	checkArray(int[][].class);
+    	checkArray(float[][].class);
+    	checkArray(long[][].class);
+    	checkArray(double[][].class);
+
+    	
+    	checkArray(String[][][].class);
+    	checkArray(boolean[][][].class);
+    	checkArray(byte[][][].class);
+    	checkArray(short[][][].class);
+    	checkArray(char[][][].class);
+    	checkArray(int[][][].class);
+    	checkArray(float[][][].class);
+    	checkArray(long[][][].class);
+    	checkArray(double[][][].class);
+    }
+
+	private void checkArray(Class<?> arrayType) {
+		assertTrue(arrayType.isArray());
+		
+		Object array = converter.convert(null).to(arrayType);
+    	assertEquals(0, Array.getLength(array));
+    	assertTrue(arrayType.isInstance(array));
+	}
 
     @Test
     public void testLongCollectionConversion() {
@@ -403,6 +450,24 @@ public class ConverterTest {
         assertArrayEquals(ia, adapted.convert("1,2").to(int[].class));
     }
 
+    @Test
+    public void testCustomConverterChaining() {
+        ConverterBuilder cb = converter.newConverterBuilder();
+        cb.rule(Date.class,
+                (f,t) -> f instanceof String ? new Date(0) : ConverterFunction.CANNOT_HANDLE);
+        Converter c1 = cb.build();
+        assertEquals(new Date(0), c1.convert("something").to(Date.class));
+        assertEquals(new Date(0), c1.convert("foo").to(Date.class));
+
+        ConverterBuilder cb2 = c1.newConverterBuilder();
+        cb2.rule(Date.class,
+                (f,t) -> f.equals("foo") ? new Date(100000) : ConverterFunction.CANNOT_HANDLE);
+        Converter c2 = cb2.build();
+        assertEquals(new Date(0), c2.convert("something").to(Date.class));
+        assertEquals(new Date(100000), c2.convert("foo").to(Date.class));
+    }
+
+
     @Test
     public void testCustomErrorHandling() {
         ConverterFunction func = new ConverterFunction() {
@@ -458,6 +523,45 @@ public class ConverterTest {
         assertEquals(123, i.value());
     }
 
+    @Test
+    public void testMultipleCustomErrorHandling() {
+        ConverterBuilder cb1 = converter.newConverterBuilder();
+        ConverterFunction func1 = new ConverterFunction() {
+            @Override
+            public Object apply(Object obj, Type targetType) {
+                return -1;
+            }
+        };
+        cb1.errorHandler(func1);
+        Converter c1 = cb1.build();
+
+        ConverterBuilder cb2 = c1.newConverterBuilder();
+        ConverterFunction func2 = new ConverterFunction() {
+            @Override
+            public Object apply(Object obj, Type targetType) {
+                if ("hello".equals(obj)) {
+                    return 0;
+                }
+                return ConverterFunction.CANNOT_HANDLE;
+            }
+        };
+        cb2.errorHandler(func2);
+        Converter adapted = cb2.build();
+
+        assertEquals(Integer.valueOf(0), adapted.convert("hello").to(Integer.class));
+        assertEquals(Integer.valueOf(-1), adapted.convert("bye").to(Integer.class));
+    }
+
+    static class MyConverterFunction implements ConverterFunction {
+        @Override
+        public Object apply(Object obj, Type targetType) throws Exception {
+            if ("hello".equals(obj)) {
+                return 0;
+            }
+            return ConverterFunction.CANNOT_HANDLE;
+        }
+    }
+
     @Test
     public void testUUIDConversion() {
         UUID uuid = UUID.randomUUID();
@@ -671,6 +775,38 @@ public class ConverterTest {
         assertEquals(Alpha.A, e2.alpha);
     }
 
+    @Test
+    public void testDTO2Map4() {
+    	MyDefaultCtorDTOAlike dto = new MyDefaultCtorDTOAlike();
+    	dto.myProp = "myValue";
+
+    	@SuppressWarnings("rawtypes")
+    	Map m = converter.convert(dto).to(Map.class);
+    	assertEquals(1, m.size());
+    	assertEquals("myValue", m.get("myProp"));
+    }
+
+    @Test
+    public void testDTO2Map5() {
+    	MyDTO3 dto = new MyDTO3();
+    	dto.charSet = new HashSet<>(Arrays.asList('f', 'o', 'o'));
+
+    	@SuppressWarnings("rawtypes")
+    	Map m = converter.convert(dto).to(new TypeReference<Map<String, ?>>() {});
+    	assertEquals(1, m.size());
+    	assertEquals(dto.charSet, m.get("charSet"));
+
+    	m = converter.convert(dto).to(new TypeReference<Map<String, ? extends List<String>>>() {});
+    	assertEquals(1, m.size());
+
+    	List<String> list = new ArrayList<>();
+    	for (Character character : dto.charSet) {
+			list.add(String.valueOf(character));
+		}
+
+    	assertEquals(list, m.get("charSet"));
+    }
+
     @Test @SuppressWarnings({ "rawtypes", "unchecked" })
     public void testDTOFieldShadowing() {
         MySubDTO dto = new MySubDTO();
